//==============================================================================
//
// Title:       TEST_HEX
// Purpose:     A short description of the application.
//
// Created on:  24.01.2020 at 23:32:17 by Caesarion.
// Copyright:   . All Rights Reserved.
//
//==============================================================================

//==============================================================================
// Include files

#include <ansi_c.h>
#include <cvirte.h>     
#include <userint.h>
#include "TEST_HEX.h"
#include "toolbox.h"

//==============================================================================
// Constants
#define TEST_TIME 60
#define NUMBER_EQUATIONS 15
#define TOP 500
#define INTERVAL 30
#define LEFT 0
#define LEFT_INPUT 80


//==============================================================================
// Types

//==============================================================================
// Static global variables

static int panelHandle;

//==============================================================================
// Static functions

//==============================================================================
// Global variables
int timer_cntr = TEST_TIME;
enum {BIN2HEX, HEX2BIN} mode;
int data_index;
int numerics[NUMBER_EQUATIONS];

//==============================================================================
// Global functions
void fail(void);
void fill(void);

/// HIFN The main entry-point function.
int main (int argc, char *argv[])
{
    int error = 0;
    
    /* initialize and load resources */
    nullChk (InitCVIRTE (0, argv, 0));
    errChk (panelHandle = LoadPanel (0, "TEST_HEX.uir", PANEL));
    
	SetCtrlVal(panelHandle, PANEL_TIMER_OUTPUT, timer_cntr);
	for (int i = 0; i < NUMBER_EQUATIONS; i++) {
		numerics[i] = NewCtrl(panelHandle, CTRL_NUMERIC, NULL, TOP - i*INTERVAL, LEFT);
		SetCtrlAttribute(panelHandle, numerics[i], ATTR_CTRL_MODE, VAL_INDICATOR);
		SetCtrlAttribute(panelHandle, numerics[i], ATTR_DATA_TYPE, VAL_INTEGER);
		SetCtrlAttribute(panelHandle, numerics[i], ATTR_SHOW_RADIX, 1);
		SetCtrlAttribute(panelHandle, numerics[i], ATTR_TEXT_POINT_SIZE, 20);
	}
	SetCtrlAttribute(panelHandle, PANEL_NUMERIC_INPUT, ATTR_TOP, TOP);
	SetCtrlAttribute(panelHandle, PANEL_NUMERIC_INPUT, ATTR_LEFT, LEFT_INPUT);
	
    /* display the panel and run the user interface */
    errChk (DisplayPanel (panelHandle));
    errChk (RunUserInterface ());

Error:
    /* clean up */
    DiscardPanel (panelHandle);
    return 0;
}

//==============================================================================
// UI callback function prototypes

/// HIFN Exit when the user dismisses the panel.
int CVICALLBACK panelCB (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    if (event == EVENT_CLOSE)
        QuitUserInterface (0);
    return 0;
}

int CVICALLBACK timer_cb (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_TIMER_TICK:
			timer_cntr--;
			SetCtrlVal(panel, PANEL_TIMER_OUTPUT, timer_cntr);
			if (!timer_cntr) {
				fail();
			}
			break;
	}
	return 0;
}

int CVICALLBACK button_cb (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			SetCtrlAttribute(panel, PANEL_TIMER, ATTR_ENABLED, 0);
			timer_cntr = TEST_TIME;
			SetCtrlVal(panel, PANEL_TIMER_OUTPUT, timer_cntr);
			
			mode = HEX2BIN;
			data_index = 0;
			SetCtrlAttribute(panelHandle, PANEL_NUMERIC_INPUT, ATTR_TOP, TOP);
			
			for (int i = 0; i < NUMBER_EQUATIONS; i++) {
				SetCtrlAttribute(panelHandle, numerics[i], ATTR_FORMAT, VAL_HEX_FORMAT);
				SetCtrlAttribute(panelHandle, numerics[i], ATTR_PADDING, 2);
			}
			fill();
			
			SetCtrlVal(panel, PANEL_TEXTMSG, "");
			SetActiveCtrl(panel, PANEL_NUMERIC_INPUT);
			FakeKeystroke(VAL_BACKSPACE_VKEY);
			SetCtrlAttribute(panel, PANEL_NUMERIC_INPUT, ATTR_FORMAT, VAL_BINARY_FORMAT);
			ResetTimer(panel, PANEL_TIMER);
			SetCtrlAttribute(panel, PANEL_TIMER, ATTR_ENABLED, 1);
			break;
	}
	return 0;
}

int CVICALLBACK input_cb (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			int timer_enabled = 0;
			GetCtrlAttribute(panel, PANEL_TIMER, ATTR_ENABLED, &timer_enabled);
			if (!timer_enabled) {
				return 0;
			}
			int number, input;
			GetCtrlVal(panel, numerics[data_index], &number);
			GetCtrlVal(panel, PANEL_NUMERIC_INPUT, &input);
			if (number == input) { // продолжаем...
				SetCtrlVal(panel, control, 0);
				FakeKeystroke(VAL_BACKSPACE_VKEY);
				data_index++;
				if (data_index == NUMBER_EQUATIONS) {
					if (mode == HEX2BIN) { // переключаем режим
						mode = BIN2HEX;
						data_index = 0;
						SetCtrlAttribute(panelHandle, PANEL_NUMERIC_INPUT, ATTR_TOP, TOP);
						
						for (int i = 0; i < NUMBER_EQUATIONS; i++) {
							SetCtrlAttribute(panelHandle, numerics[i], ATTR_FORMAT, VAL_BINARY_FORMAT);
							SetCtrlAttribute(panelHandle, numerics[i], ATTR_PADDING, 4);
						}
						fill();
						
						SetActiveCtrl(panel, PANEL_NUMERIC_INPUT);
						SetCtrlAttribute(panel, PANEL_NUMERIC_INPUT, ATTR_FORMAT, VAL_HEX_FORMAT);
					} else { // победа
						SetCtrlAttribute(panelHandle, PANEL_TIMER, ATTR_ENABLED, 0);
						SetCtrlVal(panelHandle, PANEL_TEXTMSG, "Congratulations!");
					}
				} else { // продолжаем в том же режиме
					SetCtrlAttribute(panelHandle, PANEL_NUMERIC_INPUT, ATTR_TOP, TOP - data_index*INTERVAL);
				}
			} else {
				fail();
			}
			break;
	}
	return 0;
}

void fail(void) {
	SetCtrlAttribute(panelHandle, PANEL_TIMER, ATTR_ENABLED, 0);
	SetCtrlVal(panelHandle, PANEL_TEXTMSG, "Fail!");
}

int random_data[NUMBER_EQUATIONS];
void fill(void) {
	for (int i = 0; i < NUMBER_EQUATIONS; i++) {
		random_data[i] = i+1;
	}
	SetRandomSeed(0);
	for (int i = 0; i < NUMBER_EQUATIONS; i++) {
		int index = Random(0, NUMBER_EQUATIONS - 0.001 - i); // выбираем индекс из младшей части массива random_data
		SetCtrlVal(panelHandle, numerics[i], random_data[index]);
		// а теперь сдвигаем старшую часть массива
		for (int j = index; j < NUMBER_EQUATIONS - 1; j++) {
			random_data[j] = random_data[j+1];
		}
	}
}
